ChartMeAddon = { } ;
function ChartMeAddon:new(o)
	o = o or {}
	setmetatable(o, self)
	self.__index = self
	return o
end

function ChartMeAddon:init()
	self.const = {}
	self.const.IN_PDMG = 1
	self.const.IN_MDMG = 2
	self.const.OUT_DMG = 3
	self.const.OUT_HEAL = 4
	self.const.RAID_OUT_DMG = 5
	self.const.RAID_OUT_HEAL = 6
	self.const.TITLE_BAR_HEIGHT = 16
	self.const.STATUS_BAR_HEIGHT = 16

	self.config = {}
	self.config.debug = true
	self.config.padding = 0
	self.config.frameSizeX = 200
	self.config.frameSizeY = 80
	self.config.numEntries = 60
	self.config.updateInterval = 1
	self.config.displayMeasures = {
		self.const.IN_PDMG
		,self.const.IN_MDMG
		--,self.const.OUT_DMG
		,self.const.OUT_HEAL
	}
	self.config.measure = {}
	self.config.measure[self.const.IN_PDMG] = {
		title = "Incoming Physical",
		bgr = .5,
		bgg = .5,
		bgb = .5,
		bga = .2,
		barr = 1,
		barg = 1,
		barb = 1
	}
	self.config.measure[self.const.IN_MDMG] = {
		title = "Incoming Magical",
		bgr = .5,
		bgg = .5,
		bgb = .5,
		bga = .2,
		barr = 1,
		barg = 1,
		barb = 0
	}
	self.config.measure[self.const.OUT_HEAL] = {
		title = "Self-Heal",
		bgr = .5,
		bgg = .5,
		bgb = .5,
		bga = .2,
		barr = 0,
		barg = 1,
		barb = 0
	}
	self.config.measure[self.const.OUT_DMG] = {
		title = "Damage",
		bgr = .5,
		bgg = .5,
		bgb = .5,
		bga = .2,
		barr = 1,
		barg = 0,
		barb = 0
	}
	self.config.measure[self.const.RAID_OUT_HEAL] = {
		title = "Raid Heal",
		bgr = .5,
		bgg = .5,
		bgb = .5,
		bga = .2,
		barr = 1,
		barg = 0,
		barb = 1
	}
	self.config.measure[self.const.RAID_OUT_DMG] = {
		title = "Raid Damage",
		bgr = .5,
		bgg = .5,
		bgb = .5,
		bga = .2,
		barr = 0,
		barg = 1,
		barb = 1
	}

	self.config.lowerAlpha = 0.2
	self.config.upperAlpha = 1

	self.recentLog = RecentLog:new()
	self.recentLog:init(self.config.numEntries)
	self.totalElapsed = 0
	self.myGuid = UnitGUID("player")
	self.mainFrame = CreateFrame("Frame", "Frame", UIParent)
	self.mainFrame:SetAllPoints()
	self.mainFrame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	
	self.cmInterfacePanel = CMInterfacePanel:new()
	self.cmInterfacePanel:init(self, self.config)

	self.graphFrames = { }


	self:redrawGraphFrames()
	
	self:resetMeasures()



	local me = self

	--on event is for the combat log listener	
	self.mainFrame:SetScript("OnEvent", function(self, event, ...)
		local timestamp, type, hideCaster, sourceGUID, sourceName, sourceFlags, sourceFlags2, destGUID,
			destName, destFlags, destFlags2, spellIdOrSwingDmgAmount, spellName, spellSchool, amount = select(1, ...)
		me:recordEvent(
			timestamp, type, hideCaster, sourceGUID, sourceName, sourceFlags, sourceFlags2, destGUID, 
			destName, destFlags, destFlags2, spellIdOrSwingDmgAmount, spellName, spellSchool, amount
		);
	end)

	-- timer event - to refresh every 0.5 secs
	self.mainFrame:SetScript("OnUpdate", function(self, elapsed)
		me:update(elapsed)
	end)



end

function ChartMeAddon:redrawGraphFrames()
	local offset = 200
	for measure, _ in pairs(self.config.measure) do

		--print ('init frames for measure '..measure)
		if self.graphFrames[measure] == nil then
			self.graphFrames[measure] = CMGraphFrame:new()
			local conf = self.config.measure[measure]
			self.graphFrames[measure]:init(
				self.const, self.config, measure, conf.title, conf.bgr, conf.bgg, conf.bgb, conf.bga,
				conf.barr, conf.barg, conf.barb, self.cmInterfacePanel, offset
			)
		end
		if ChartMeAddon.inTable(measure, self.config.displayMeasures) then
			self.graphFrames[measure]:show()
			offset = offset + self.config.frameSizeY+self.const.TITLE_BAR_HEIGHT
		else
			self.graphFrames[measure]:hide()
		end
		
	end
end

-- static --
function ChartMeAddon.dump(o)
   if type(o) == 'table' then
      local s = '{ '
      for k,v in pairs(o) do
         if type(k) ~= 'number' then k = '"'..k..'"' end
         s = s .. '['..k..'] = ' .. ChartMeAddon.dump(v) .. ','
      end
      return s .. '} '
   else
      return tostring(o)
   end
end

-- static --
function ChartMeAddon.inTable(needle, haystack)
  local u = { }
  for _, v in pairs(haystack) do u[v] = true end
  if u[needle] then return true end
  return nil
end

function ChartMeAddon:addMeasure(type, amount)
	--print ("adding ".. amount .. " to measure ".. type)
	self.eventData[type] = self.eventData[type] + amount
end
function ChartMeAddon:getMeasure(type)
	return self.eventData[type]
end
function ChartMeAddon:resetMeasures()
	self.eventData = {};
	for measure, _ in pairs(self.config.measure) do
		self.eventData[measure] = 0;
	end
end

function ChartMeAddon:getEventDataByType(type)
	local typedEntries = {}
	if self.recentLog then

		local entries = self.recentLog:getAllEvents()
		
		local count = self.recentLog:getCount()

		for i = 0, (count-1) do
			typedEntries[i] = entries[i][type]
		end
	end
	return typedEntries
end

function ChartMeAddon:refreshData()

	if self.recentLog then
		
		local count = self.recentLog:getCount()

		--print(self:dump(self.graphFrames[self.const.OUT_HEAL]))
		
		for measure, _ in pairs(self.config.measure) do
			local graphFrame = self.graphFrames[measure]
			local entries = self:getEventDataByType(measure)
			--print(self:dump(entries))
			graphFrame:plotDmgGraph(entries, measure, count)
		end

		--print ('count '..count)
	end
end

function ChartMeAddon:recordEvent(
	timestamp, type, hideCaster, sourceGUID, sourceName, sourceFlags, sourceRaidFlags, destGUID,
	destName, destFlags, destRaidFlags, spellId, spellName, spellSchool, amount,
	overkill, school, resisted, blocked, absorbed
)
	--print "event recording"
	--the people at blizz decided to not include the spellId, spellName and spellSchool if the dmg is a melee swing
	if type == 'SWING_DAMAGE' then
		absorbed = school
		blocked = overkill
		resisted = amount
		school = spellSchool
		overkill = spellName
		amount = spellId
	end
	for measure, _ in pairs(self.config.measure) do
		local valToStore = 0;
		if measure == self.const.IN_PDMG then
			if destGUID == self.myGuid then
				--[[
				local str = '';
				if (absorbed) then
					str = str .. " absorbed: "..absorbed
				end
				if (blocked) then
					str = str .. " blocked: "..blocked
				end
				if (resisted) then
					str = str .. " resisted: "..resisted
				end
				if (amount) then
					str = str .. " AMOUNT: "..amount
				end
				print (str)
				]]--
				if string.match(type, "_DAMAGE") then
					--swing dmg
					if type == 'SWING_DAMAGE' then
						valToStore = amount
					--or if its physical add to physical dmg
					elseif spellSchool == 1 then
						valToStore = amount
					end
				end
			end
		elseif measure == self.const.IN_MDMG then
			if destGUID == chartMeAddon.myGuid then
				if string.match(type, "_DAMAGE") then
					-- if its not swing_damage and not physical then add to magical dmg
					if type ~= "SWING_DAMAGE" and spellSchool ~= 1 then
						valToStore = amount
					end
				end
			end
		end
		if measure == self.const.OUT_DMG then
			if sourceGUID == self.myGuid then
				--print "i did something"
				if string.match(type, "_DAMAGE") then
					--print ("i did some dmg")
					valToStore = amount
					if (self.config.debug) then
						--print ("i did some dmg " .. valToStore)
					end
				end
			end
		end
		if measure == self.const.OUT_HEAL then
			if sourceGUID == self.myGuid then
				if string.match(type, "_HEAL") then
					valToStore = amount;
				end
			end
		end
		if measure == self.const.RAID_OUT_DMG then
			if
				bit.band(sourceFlags, COMBATLOG_OBJECT_AFFILIATION_RAID) or
				bit.band(sourceFlags, COMBATLOG_OBJECT_AFFILIATION_PARTY) or
				sourceGUID == self.myGuid
			then
				if string.match(type, "_DAMAGE") then
					valToStore = amount
				end
			end
		end
		if measure == self.const.RAID_OUT_HEAL then
			if bit.band(sourceFlags, COMBATLOG_OBJECT_AFFILIATION_RAID) then
				if string.match(type, "_HEAL") then
					valToStore = amount
				end
			end
		end
		if valToStore > 0 then
			if (self.config.debug) then
				--print("adding "..valToStore.." to measure "..measure)
			end
			self:addMeasure(measure, valToStore)
		end
	end
end

function ChartMeAddon:pauseInit(delay)
	self.pauseInitTotalElapsed = 0
	if(type(delay)~="number") then
		return false;
	end
	local waitFrame = CreateFrame("Frame","WaitFrame", UIParent)
	local me = self
	waitFrame:SetScript("onUpdate",function (self,elapse)
		me.pauseInitTotalElapsed = me.pauseInitTotalElapsed + elapse
		if (me.pauseInitTotalElapsed > 5) then
			if (me.config == nil) then
				me:init()
			end
		end
	end)
	return true
end

function ChartMeAddon:update(elapsed)
	self.totalElapsed = self.totalElapsed + elapsed
  	if self.totalElapsed > self.config.updateInterval then
    	self.totalElapsed = 0;
    	self.recentLog:add(self.eventData);
    	if self.config.debug then
	    	--print (self:getMeasure(self.const.OUT_HEAL))
	    end
    	self:refreshData()
	--ChartMeAddon["recentLog"]:display()
    	self:resetMeasures()
  end
end

chartMeAddon = ChartMeAddon:new()
chartMeAddon:pauseInit(5)


